var Closure = require("bn-lang").Closure;

var TimerTask = function(){this.init.apply(this, arguments);};
exports.TimerTask = TimerTask;

/**
* @class TimerTask
* @constructor
* @param {Function} _block
* @param {Object} _object
* @param {Array} _args
* @param {Number} _runInterval
*/
TimerTask.prototype.init = function(_block, _object, _args, _runInterval) {

	this.block = _block;
	this.object = _object;
	this.args = _args;

	this.tickCount = _runInterval;

	this.currentTick = this.tickCount;
	this.running = false;
};

TimerTask.prototype.start = function() {

	this.running = true;
};

TimerTask.prototype.stop = function() {

	this.running = false;
};

TimerTask.prototype.run = function() {

	this.currentTick--;
	if (this.currentTick <= 0) {

		if(this.running) {

			new Closure().bind(this.block, this.object, this.args || [])();
		}

		// reset the this.currentTick from the original this.tickCount
		this.currentTick = this.tickCount;
	}
};

TimerTask.prototype.isRunning = function() {

	return this.running;
};
var AsyncTask = require("bn-lang").AsyncTask;

/**
* @class Timer
* @constructor
*/
var Timer = function(){this.init.apply(this, arguments);};
exports.Timer = Timer;

Timer.prototype.init = function() {

	this.tasks = [];
	this.running = false;

	this.daemonAsyncTask = new AsyncTask(function() {

		this.tasks.each(function(_task) {

			_task.run();
		});
	}, this, [], Timer.INTERVAL, "Timer Daemon AsyncTask");
	this.daemonAsyncTask.setDaemon(true);
};

Timer.prototype.start = function() {

	if(!this.running) {

		this.running = true;
		this.daemonAsyncTask.start();
	}
};

Timer.prototype.stop = function() {

	if(this.running) {

		this.running = false;
		this.daemonAsyncTask.stop();
	}
};

Timer.prototype.pause = function(_length) {

	if(this.running) {

		this.stop();
		this.daemonAsyncTask.sleep(_length);
	}
};

Timer.prototype.pauseResume = function() {

	if(this.running) {

		this.stop();

	} else {

		this.start();
	}
};

Timer.prototype.getTaskByReference = function(_object, _block) {

	return this.tasks.find(function(_task) {

		if(_task.object === _object && _task.block === _block) {

			return true;
		}
	});
};

Timer.prototype.addTask = function(_object, _block, _args, _runInterval) {

	var task = this.getTaskByReference(_object, _block);

	if(!task) {

		this.tasks.add(new TimerTask(_block, _object, _args, Number.parseInt(_runInterval / Timer.INTERVAL)));
	}
};

Timer.prototype.deleteTask = function(_object, _block) {

	var task = this.getTaskByReference(_object, _block);

	if(task) {

		task.stop();
		this.tasks.remove(task);
	}
};

Timer.prototype.startTask = function(_object, _block) {

	if(this.running) {

		var task = this.getTaskByReference(_object, _block);
		if(task) {

			task.start();
		}
	}
};

Timer.prototype.stopTask = function(_object, _block) {

	if(this.running) {

		var task = this.getTaskByReference(_object, _block);
		if(task) {

			task.stop();
		}
	}
};

Timer.prototype.pauseTask = function(_object, _block, _length) {

	if(this.running) {

		var task = this.getTaskByReference(_object, _block);
		if(task && task.running) {

			task.stop();
			new AsyncTask(function() {

				task.start();
			}, this, [], _length).start();
		}
	}
};

Timer.prototype.isRunning = function() {

	return this.running;
};

Timer.INTERVAL = 16; // milliseconds
